@@ -10,6 +10,7 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+from concurrent.futures import CancelledError
 
 
 class RetriesExceededError(Exception):
@@ -24,3 +25,8 @@ class S3UploadFailedError(Exception):
 
 class InvalidSubscriberMethodError(Exception):
     pass
+
+
+class FatalError(CancelledError):
+    """A CancelledError raised from an error in the TransferManager"""
+    pass
